/*
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "daap-music.h"
#include "dmap-connection.h"
#include "dmap-structure.h"
#include "dmap-marshal.h"

#include <string.h>

struct _DAAPMusicPrivate {
    gchar *album;
    gchar *artist;
    gchar *format;
    gchar *genre;
    gint length;
    gint track_number;
    gint disc_number;
    gint year;
    gint size;
    gint bitrate;
};

enum {
    PROP_0,
    PROP_ALBUM,
    PROP_ARTIST,
    PROP_FORMAT,
    PROP_GENRE,
    PROP_LENGTH,
    PROP_TRACK_NUMBER,
    PROP_DISC_NUMBER,
    PROP_YEAR,
    PROP_SIZE,
    PROP_BITRATE,
    PROP_URI
};

static void daap_music_dispose (GObject *obj);
static void daap_music_set_property (GObject *object,
                                     guint prop_id,
                                     const GValue *value,
                                     GParamSpec *pspec);
static void daap_music_get_property (GObject *object,
                                     guint prop_id,
                                     GValue *value,
                                     GParamSpec *pspec);

G_DEFINE_TYPE (DAAPMusic, daap_music, TYPE_DMAP_ITEM)

#define DAAP_MUSIC_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DAAP_MUSIC, DAAPMusicPrivate))

static void
daap_music_class_init (DAAPMusicClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->dispose = daap_music_dispose;
    object_class->set_property = daap_music_set_property;
    object_class->get_property = daap_music_get_property;

    g_type_class_add_private (klass, sizeof (DAAPMusicPrivate));

    g_object_class_install_property (object_class,
            PROP_ALBUM,
            g_param_spec_string ("album",
                "self album",
                "self album",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_ARTIST,
            g_param_spec_string ("artist",
                "self artist",
                "self artist",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_FORMAT,
            g_param_spec_string ("format",
                "self format",
                "self format",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_GENRE,
            g_param_spec_string ("genre",
                "self genre",
                "self genre",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_LENGTH,
            g_param_spec_int ("length",
                "self length",
                "self length",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_TRACK_NUMBER,
            g_param_spec_int ("track-number",
                "self track number",
                "self track number",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_DISC_NUMBER,
            g_param_spec_int ("disc-number",
                "self disc number",
                "self disc number",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_YEAR,
            g_param_spec_int ("year",
                "self year",
                "self year",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_SIZE,
            g_param_spec_int ("size",
                "self size",
                "self size",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_BITRATE,
            g_param_spec_int ("bitrate",
                "self bitrate",
                "self bitrate",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_URI,
            g_param_spec_string ("uri",
                "self uri",
                "self uri",
                NULL,
                G_PARAM_READABLE));
}

static void
daap_music_init (DAAPMusic *self)
{
    self->priv = DAAP_MUSIC_GET_PRIVATE (self);
}

static void
daap_music_dispose (GObject *object)
{
    DAAPMusicPrivate *priv = DAAP_MUSIC (object)->priv;

    g_free (priv->album);
    g_free (priv->artist);
    g_free (priv->format);
    g_free (priv->genre);

    G_OBJECT_CLASS (daap_music_parent_class)->dispose (object);
}

static void
daap_music_set_property (GObject *object,
                         guint prop_id,
                         const GValue *value,
                         GParamSpec *pspec)
{
    DAAPMusicPrivate *priv = DAAP_MUSIC (object)->priv;

    switch (prop_id) {
    case PROP_ALBUM:
        priv->album = g_value_dup_string (value);
        break;
    case PROP_ARTIST:
        priv->artist = g_value_dup_string (value);
        break;
    case PROP_FORMAT:
        priv->format = g_value_dup_string (value);
        break;
    case PROP_GENRE:
        priv->genre = g_value_dup_string (value);
        break;
    case PROP_LENGTH:
        priv->length = g_value_get_int (value);
        break;
    case PROP_TRACK_NUMBER:
        priv->track_number = g_value_get_int (value);
        break;
    case PROP_DISC_NUMBER:
        priv->disc_number = g_value_get_int (value);
        break;
    case PROP_YEAR:
        priv->year = g_value_get_int (value);
        break;
    case PROP_SIZE:
        priv->size = g_value_get_int (value);
        break;
    case PROP_BITRATE:
        priv->bitrate = g_value_get_int (value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

static void
daap_music_get_property (GObject *object,
                         guint prop_id,
                         GValue *value,
                         GParamSpec *pspec)
{
    DAAPMusicPrivate *priv = DAAP_MUSIC (object)->priv;

    switch (prop_id) {
    case PROP_ALBUM:
        g_value_set_string (value, priv->album);
        break;
    case PROP_ARTIST:
        g_value_set_string (value, priv->artist);
        break;
    case PROP_FORMAT:
        g_value_set_string (value, priv->format);
        break;
    case PROP_GENRE:
        g_value_set_string (value, priv->genre);
        break;
    case PROP_LENGTH:
        g_value_set_int (value, priv->length);
        break;
    case PROP_TRACK_NUMBER:
        g_value_set_int (value, priv->track_number);
        break;
    case PROP_DISC_NUMBER:
        g_value_set_int (value, priv->disc_number);
        break;
    case PROP_YEAR:
        g_value_set_int (value, priv->year);
        break;
    case PROP_SIZE:
        g_value_set_int (value, priv->size);
        break;
    case PROP_BITRATE:
        g_value_set_int (value, priv->bitrate);
        break;
    case PROP_URI:
        {
            DMAPConnection *connection;
            gchar *username = NULL, *password = NULL, *host;
            gchar *uri;
            gint port, id, database_id, session_id;
	    gdouble protocol_version;

            g_object_get (object,
                    "connection", &connection,
                    "id", &id,
                    NULL);
            g_object_get (G_OBJECT (connection),
                    "host", &host,
                    "port", &port,
                    "username", &username,
                    "password", &password,
		    "database-id", &database_id,
		    "session-id", &session_id,
		    "protocol_version", &protocol_version,
                    NULL);
            if (username && password) {
                uri = g_strdup_printf ("daap://%s:%s@%s:%d/databases/%d/items/%d.%s?session-id=%u&protocol-version=%.2f", username, password, host, port, database_id, 
                        id, priv->format, session_id, protocol_version);
                g_free (username);
                g_free (password);
            }
            else {
                uri = g_strdup_printf ("daap://%s:%d/databases/%d/items/%d.%s?session-id=%u&protocol-version=%.2f", host, port, database_id, id, priv->format, session_id, protocol_version);
            }
            g_free (host);
            g_object_unref (connection);
            g_value_set_string (value, uri);
            g_free (uri);
        }
        break;

    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}
