/*
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "daap-playlist.h"
#include "daap-music.h"
#include "dmap-connection.h"
#include "dmap-structure.h"
#include "dmap-marshal.h"

#include <string.h>

static gboolean daap_playlist_retrieve_musics (DMAPItem *self);

G_DEFINE_TYPE (DAAPPlaylist, daap_playlist, TYPE_DMAP_ITEM)

static void
daap_playlist_class_init (DAAPPlaylistClass *klass)
{
    DMAPItemClass *dmap_item_class = (DMAPItemClass *) klass;
    dmap_item_class->retrieve_items = daap_playlist_retrieve_musics;
}

static void
daap_playlist_init (DAAPPlaylist *self)
{
}

static void
handle_playlist_entries (DMAPConnection *connection,
                         guint status,
                         GNode *structure,
                         DMAPItem *self)
{
    GNode *listing_node;
    GNode *node;
    gint i;
    DMAPItem *parent, *music;

    if (structure == NULL || SOUP_STATUS_IS_SUCCESSFUL (status) == FALSE) {
        return;
    }

    listing_node = dmap_structure_find_node (structure, DMAP_CC_MLCL);
    if (listing_node == NULL) {
        return;
    }

    parent = dmap_item_get_parent (self);
    for (i = 0, node = listing_node->children; node; node = node->next, i++) {
        DMAPStructureItem *item;
        GNode *n2;
        gint item_id = 0;
        const gchar *title = NULL;
        const gchar *album = NULL;
        const gchar *artist = NULL;
        const gchar *format = NULL;
        const gchar *genre = NULL;
        gint length = 0;
        gint track_number = 0;
        gint disc_number = 0;
        gint year = 0;
        gint size = 0;
        gint bitrate = 0;

        item = dmap_structure_find_item (node, DMAP_CC_MIID);
        if (item == NULL) {
            g_debug ("Could not find dmap.itemid item in /databases");
            continue;
        }
        item_id = g_value_get_int (&(item->content));

        if (parent)
            music = dmap_item_get_child (parent, item_id);
        else
            music = NULL;

        if (!music) {
            for (n2 = node->children; n2; n2 = n2->next) {
                DMAPStructureItem *meta_item;

                meta_item = n2->data;

                switch (meta_item->content_code) {
                    case DMAP_CC_MINM:
                        title = g_value_get_string (&(meta_item->content));
                        break;
                    case DMAP_CC_ASAL:
                        album = g_value_get_string (&(meta_item->content));
                        break;
                    case DMAP_CC_ASAR:
                        artist = g_value_get_string (&(meta_item->content));
                        break;
                    case DMAP_CC_ASFM:
                        format = g_value_get_string (&(meta_item->content));
                        break;
                    case DMAP_CC_ASGN:
                        genre = g_value_get_string (&(meta_item->content));
                        break;
                    case DMAP_CC_ASTM:
                        length = g_value_get_int (&(meta_item->content));
                        break;
                    case DMAP_CC_ASTN:
                        track_number = g_value_get_int (&(meta_item->content));
                        break;
                    case DMAP_CC_ASDN:
                        disc_number = g_value_get_int (&(meta_item->content));
                        break;
                    case DMAP_CC_ASYR:
                        year = g_value_get_int (&(meta_item->content));
                        break;
                    case DMAP_CC_ASSZ:
                        size = g_value_get_int (&(meta_item->content));
                        break;
                    case DMAP_CC_ASBR:
                        bitrate = g_value_get_int (&(meta_item->content));
                        break;
                    default:
                        break;
                }
            }

            music = DMAP_ITEM (g_object_new (TYPE_DAAP_MUSIC,
                    "connection", connection,
                    "id", item_id,
                    "name", title,
                    "album", album,
                    "artist", artist,
                    "format", format,
                    "genre", genre,
                    "length", length,
                    "track-number", track_number,
                    "disc-number", disc_number,
                    "year", year,
                    "size", size,
                    "bitrate", bitrate,
                    NULL));

            dmap_item_add_child (self, music);
        }
        else {
            dmap_item_add_child (self, music);
        }
    }

    dmap_item_signal_loaded (self);
}

static gboolean
daap_playlist_retrieve_musics (DMAPItem *self)
{
    DMAPConnection *connection;
    DMAPItem *parent;
    gint id, database_id, session_id, revision_number;
    gchar *path;
    gboolean ret;

    g_debug ("retrieving musics from playlist");

    g_object_get (G_OBJECT (self),
            "connection", &connection,
            "id", &id,
            "parent", &parent,
	    "database-id", &database_id,
	    "session-id", &session_id,
	    "revision-number", &revision_number,
            NULL);

    if (!parent) {
        path = g_strdup_printf ("/databases/%d/items?session-id=%d&revision-number=%d"
                "&meta=dmap.itemid,dmap.itemname,daap.songplaylist,"
                "daap.songalbum,daap.songartist,daap.songgenre,daap.songsize,"
                "daap.songtime,daap.songtrackcount,daap.songtracknumber,"
                "daap.songyear,daap.songformat,"
                "daap.songbitrate",
		database_id,
		session_id,
    		revision_number);
    }
    else {
        path = g_strdup_printf ("/databases/%d/containers/%d/items?session-id=%d&revision-number=%d"
                "&meta=dmap.itemid",
		database_id,
                id,
		session_id,
		revision_number);
        g_object_unref (parent);
    }
    ret = dmap_connection_get (connection, path,
            TRUE, (DMAPResponseHandler) handle_playlist_entries, self);
    g_free (path);
    g_object_unref (connection);

    return ret;
}
