/*
 * Copyright (C) 2004,2005 Charles Schmidt <cschmidt2@emich.edu>
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "daap-root-item.h"
#include "daap-playlist.h"
#include "dmap-connection.h"
#include "dmap-structure.h"

#include <libsoup/soup.h>
#include <libsoup/soup-session-sync.h>
#include <libsoup/soup-uri.h>

static gboolean daap_root_item_retrieve_playlists (DMAPItem *self);
static void root_playlist_loaded (DMAPItem *playlist,
                                  DMAPItem *self);
static void handle_playlists (DMAPConnection *connection,
                              guint status,
                              GNode *structure,
                              DMAPItem *self);

G_DEFINE_TYPE (DAAPRootItem, daap_root_item, TYPE_DMAP_ITEM)

static void
daap_root_item_class_init (DAAPRootItemClass *klass)
{
    DMAPItemClass *dmap_item_class = (DMAPItemClass *) klass;

    dmap_item_class->retrieve_items = daap_root_item_retrieve_playlists;
}

static void
daap_root_item_init (DAAPRootItem *self)
{
}

static gboolean
daap_root_item_retrieve_playlists (DMAPItem *self)
{
    DMAPConnection *connection;
    gchar *path;
    gint database_id, session_id, revision_number;
    gboolean ret;

    g_object_get (G_OBJECT (self),
            "connection", &connection,
	    "database-id", &database_id,
	    "session-id", &session_id,
	    "revision-number", &revision_number,
            NULL);

    path = g_strdup_printf ("/databases/%d/containers?session-id=%d&revision-number=%d", database_id, session_id, revision_number);
    ret = dmap_connection_get (connection, path,
            TRUE, (DMAPResponseHandler) handle_playlists, self);
    g_free (path);
    g_object_unref (connection);

    return ret;
}

static void
root_playlist_loaded (DMAPItem *playlist,
                      DMAPItem *self)
{
    g_signal_handlers_disconnect_by_func (playlist,
            G_CALLBACK (root_playlist_loaded),
            self);
    dmap_item_signal_loaded (self);
}

static void
handle_playlists (DMAPConnection *connection,
                  guint status,
                  GNode *structure,
                  DMAPItem *self)
{
    GNode *listing_node;
    gint i;
    GNode *n;
    DMAPItem *root_playlist, *playlist;

    if (structure == NULL || SOUP_STATUS_IS_SUCCESSFUL (status) == FALSE) {
        return;
    }

    listing_node = dmap_structure_find_node (structure, DMAP_CC_MLCL);
    if (listing_node == NULL) {
	gint database_id;
	g_object_get (connection, "database-id", &database_id, NULL);
        g_debug ("Could not find dmap.listing item in /databases/%d/containers",
		 database_id);
        return;
    }

    root_playlist = DMAP_ITEM (g_object_new (TYPE_DAAP_PLAYLIST,
            "connection", connection,
            "id", -1,
            "name", "All",
            "is-folder", TRUE,
            NULL));
    g_signal_connect (root_playlist,
                      "loaded",
                      G_CALLBACK (root_playlist_loaded),
                      self);
    dmap_item_add_child (self, root_playlist);

    for (i = 0, n = listing_node->children; n; n = n->next, i++) {
        DMAPStructureItem *item;
        gint item_id;
        const gchar *name;

        item = dmap_structure_find_item (n, DMAP_CC_ABPL);
        if (item != NULL) {
            continue;
        }

        item = dmap_structure_find_item (n, DMAP_CC_MIID);
        if (item == NULL) {
	    gint database_id;
	    g_object_get (connection, "database_id", &database_id, NULL);
            g_debug ("Could not find dmap.itemid item in /databases/%d/containers",
	    	     database_id);
            continue;
        }
        item_id = g_value_get_int (&(item->content));

        item = dmap_structure_find_item (n, DMAP_CC_MINM);
        if (item == NULL) {
	    gint database_id;
	    g_object_get (connection, "database_id", &database_id, NULL);
            g_debug ("Could not find dmap.itemname item in /databases/%d/containers",
	    	     database_id);
            continue;
        }
        name = g_value_get_string (&(item->content));

        g_debug ("Got playlist with id %d, name %s", item_id, name);

        playlist = DMAP_ITEM (g_object_new (TYPE_DAAP_PLAYLIST,
                "connection", connection,
                "id", item_id,
                "name", name,
                "parent", root_playlist,
                "is-folder", TRUE,
                NULL));
        dmap_item_add_child (self, playlist);
    }

    dmap_item_retrieve_items (root_playlist, FALSE);
}
