/*
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "dmap-item.h"
#include "dmap-connection.h"

#include <string.h>

struct _DMAPItemPrivate {
    DMAPConnection *connection;
    gint id;
    gchar *name;
    DMAPItem *parent;
    gboolean is_folder;
    GSList *items;
    GHashTable *hash_items;
    guint emit_loaded_source;
};

enum {
    PROP_0,
    PROP_CONNECTION,
    PROP_ID,
    PROP_NAME,
    PROP_PARENT,
    PROP_IS_FOLDER,
};

enum {
    LOADED,
    LAST_SIGNAL
};

static void dmap_item_dispose (GObject *obj);
static void dmap_item_finalize (GObject *obj);
static void dmap_item_set_property (GObject *object,
                                     guint prop_id,
                                     const GValue *value,
                                     GParamSpec *pspec);
static void dmap_item_get_property (GObject *object,
                                     guint prop_id,
                                     GValue *value,
                                     GParamSpec *pspec);
static gboolean dmap_item_remove_item (gpointer key,
                                       gpointer value,
                                       gpointer user_data);

static gboolean dmap_item_emit_loaded (DMAPItem *item);

G_DEFINE_ABSTRACT_TYPE (DMAPItem, dmap_item, G_TYPE_OBJECT)

#define DMAP_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DMAP_ITEM, DMAPItemPrivate))

static guint dmap_item_signals[LAST_SIGNAL] = { 0, };

static void
dmap_item_class_init (DMAPItemClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->dispose = dmap_item_dispose;
    object_class->finalize = dmap_item_finalize;
    object_class->set_property = dmap_item_set_property;
    object_class->get_property = dmap_item_get_property;

    g_type_class_add_private (klass, sizeof (DMAPItemPrivate));

    g_object_class_install_property (object_class,
            PROP_CONNECTION,
            g_param_spec_object ("connection",
                "item connection",
                "item connection",
                TYPE_DMAP_CONNECTION,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_ID,
            g_param_spec_int ("id",
                "item id",
                "item id",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_NAME,
            g_param_spec_string ("name",
                "item name",
                "item name",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_PARENT,
            g_param_spec_object ("parent",
                "parent item",
                "parent item",
                TYPE_DMAP_ITEM,
                G_PARAM_READWRITE /* | G_PARAM_CONSTRUCT */));
    g_object_class_install_property (object_class,
            PROP_IS_FOLDER,
            g_param_spec_boolean ("is-folder",
                "whether the item can contain child items",
                "whether the item can contain child items",
                FALSE,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));

    dmap_item_signals [LOADED] = g_signal_new ("loaded",
            G_TYPE_FROM_CLASS (object_class),
            G_SIGNAL_RUN_LAST,
            G_STRUCT_OFFSET (DMAPItemClass, loaded),
            NULL,
            NULL,
            g_cclosure_marshal_VOID__VOID,
            G_TYPE_NONE,
            0);
}

static void
dmap_item_init (DMAPItem *self)
{
    self->priv = DMAP_ITEM_GET_PRIVATE (self);
    self->priv->hash_items = g_hash_table_new_full (g_direct_hash, g_direct_equal,
            NULL, g_object_unref);
}

static void
dmap_item_dispose (GObject *object)
{
    DMAPItemPrivate *priv = DMAP_ITEM (object)->priv;

    if (priv->emit_loaded_source)
        g_source_remove (priv->emit_loaded_source);

    g_hash_table_destroy (priv->hash_items);
    g_slist_free (priv->items);
    g_free (priv->name);

    G_OBJECT_CLASS (dmap_item_parent_class)->dispose (object);
}

static void
dmap_item_finalize (GObject *object)
{
    g_signal_handlers_destroy (object);
    G_OBJECT_CLASS (dmap_item_parent_class)->finalize (object);
}

static void
dmap_item_set_property (GObject *object,
                        guint prop_id,
                        const GValue *value,
                        GParamSpec *pspec)
{
    DMAPItemPrivate *priv = DMAP_ITEM (object)->priv;

    switch (prop_id) {
    case PROP_CONNECTION:
        priv->connection = DMAP_CONNECTION (g_value_get_object (value));
        break;
    case PROP_ID:
        priv->id = g_value_get_int (value);
        break;
    case PROP_NAME:
        priv->name = g_value_dup_string (value);
        break;
    case PROP_PARENT:
        priv->parent = DMAP_ITEM (g_value_get_object (value));
        break;
    case PROP_IS_FOLDER:
        priv->is_folder = g_value_get_boolean (value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

static void
dmap_item_get_property (GObject *object,
                        guint prop_id,
                        GValue *value,
                        GParamSpec *pspec)
{
    DMAPItemPrivate *priv = DMAP_ITEM (object)->priv;

    switch (prop_id) {
    case PROP_CONNECTION:
        g_value_set_object (value, priv->connection);
        break;
    case PROP_ID:
        g_value_set_int (value, priv->id);
        break;
    case PROP_NAME:
        g_value_set_string (value, priv->name);
        break;
    case PROP_PARENT:
        g_value_set_object (value, priv->parent);
        break;
    case PROP_IS_FOLDER:
        g_value_set_boolean (value, priv->is_folder);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

gboolean
dmap_item_retrieve_items (DMAPItem *self,
                          gboolean force_reload)
{
    DMAPItemClass *klass;

    g_return_val_if_fail (self != NULL, FALSE);
    klass = DMAP_ITEM_GET_CLASS (self);
    g_return_val_if_fail (klass->retrieve_items != NULL, FALSE);

    if (self->priv->items && !force_reload) {
        if (!self->priv->emit_loaded_source)
            self->priv->emit_loaded_source = g_idle_add (
                (GSourceFunc) dmap_item_emit_loaded, (gpointer) self);
        return TRUE;
    }

    if (force_reload && self->priv->hash_items) {
        g_hash_table_foreach_remove (self->priv->hash_items, dmap_item_remove_item, NULL);
        /* do not use g_hash_table_remove_all. not supported on 770 glib version */
        /* g_hash_table_remove_all (self->priv->hash_items); */
    }

    return klass->retrieve_items (self);
}

const GSList *
dmap_item_get_items (DMAPItem *self)
{
    g_return_val_if_fail (self != NULL, NULL);
    return self->priv->items;
}

DMAPItem *
dmap_item_get_parent (DMAPItem *self)
{
    g_return_val_if_fail (self != NULL, NULL);
    if (self->priv->parent)
        return g_object_ref (self->priv->parent);
    return NULL;
}

void
dmap_item_add_child (DMAPItem *self,
                     DMAPItem *child)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (child != NULL);

    if (g_hash_table_lookup (self->priv->hash_items, GUINT_TO_POINTER (child->priv->id))) {
        g_debug ("item with same id %d on the same playlist. ignoring!", child->priv->id);
        g_object_unref (child);
        return;
    }
    g_hash_table_insert (self->priv->hash_items,
            GUINT_TO_POINTER (child->priv->id), child);
    self->priv->items = g_slist_append (self->priv->items, child);
}

DMAPItem *
dmap_item_get_child (DMAPItem *self,
                     guint id)
{
    DMAPItem *item;

    g_return_val_if_fail (self != NULL, NULL);

    item = g_hash_table_lookup (self->priv->hash_items, GUINT_TO_POINTER (id));
    if (item) {
        return g_object_ref (item);
    }
    return NULL;
}

void
dmap_item_signal_loaded (DMAPItem *self)
{
    g_return_if_fail (self != NULL);
    g_signal_emit (self, dmap_item_signals [LOADED], 0);
}

static gboolean
dmap_item_emit_loaded (DMAPItem *self)
{
    dmap_item_signal_loaded (self);
    self->priv->emit_loaded_source = 0;
    return FALSE;
}

/* HACK */
static gboolean
dmap_item_remove_item (gpointer key,
                       gpointer value,
                       gpointer user_data)
{
    return TRUE;
}
