/*
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA*
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "dpap-photo.h"
#include "dpap-connection.h"
#include "dmap-structure.h"
#include "dmap-marshal.h"

#include <string.h>

struct _DPAPPhotoPrivate {
    gchar *aspect_ratio;
    gint creation_date;
};

enum {
    PROP_0,
    PROP_ASPECT_RATIO,
    PROP_CREATION_DATE,
    PROP_URI
};

static void dpap_photo_dispose (GObject *object);
static void dpap_photo_set_property (GObject *object,
                                     guint prop_id,
                                     const GValue *value,
                                     GParamSpec *pspec);
static void dpap_photo_get_property (GObject *object,
                                     guint prop_id,
                                     GValue *value,
                                     GParamSpec *pspec);

G_DEFINE_TYPE (DPAPPhoto, dpap_photo, TYPE_DMAP_ITEM)

#define DPAP_PHOTO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DPAP_PHOTO, DPAPPhotoPrivate))

static void
dpap_photo_class_init (DPAPPhotoClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->dispose = dpap_photo_dispose;
    object_class->set_property = dpap_photo_set_property;
    object_class->get_property = dpap_photo_get_property;

    g_type_class_add_private (klass, sizeof (DPAPPhotoPrivate));

    g_object_class_install_property (object_class,
            PROP_ASPECT_RATIO,
            g_param_spec_string ("aspect-ratio",
                "self aspect ratio",
                "self aspect ratio",
                NULL,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_CREATION_DATE,
            g_param_spec_int ("creation-date",
                "self creation date",
                "self creation date",
                G_MININT, G_MAXINT, -1,
                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    g_object_class_install_property (object_class,
            PROP_URI,
            g_param_spec_string ("uri",
                "self uri",
                "self uri",
                NULL,
                G_PARAM_READABLE));
}

static void
dpap_photo_init (DPAPPhoto *self)
{
    self->priv = DPAP_PHOTO_GET_PRIVATE (self);
}

static void
dpap_photo_dispose (GObject *object)
{
    DPAPPhotoPrivate *priv = DPAP_PHOTO (object)->priv;

    g_free (priv->aspect_ratio);

    G_OBJECT_CLASS (dpap_photo_parent_class)->dispose (object);
}

static void
dpap_photo_set_property (GObject *object,
                         guint prop_id,
                         const GValue *value,
                         GParamSpec *pspec)
{
    DPAPPhotoPrivate *priv = DPAP_PHOTO (object)->priv;

    switch (prop_id) {
    case PROP_ASPECT_RATIO:
        priv->aspect_ratio = g_value_dup_string (value);
        break;
    case PROP_CREATION_DATE:
        priv->creation_date = g_value_get_int (value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

static void
dpap_photo_get_property (GObject *object,
                         guint prop_id,
                         GValue *value,
                         GParamSpec *pspec)
{
    DPAPPhotoPrivate *priv = DPAP_PHOTO (object)->priv;

    switch (prop_id) {
    case PROP_ASPECT_RATIO:
        g_value_set_string (value, priv->aspect_ratio);
        break;
    case PROP_CREATION_DATE:
        g_value_set_int (value, priv->creation_date);
        break;
    case PROP_URI:
        {
            DMAPConnection *connection;
            gchar *username = NULL, *password = NULL, *host;
            gchar *uri, *name;
            gint port, id, database_id, session_id;

            g_object_get (object,
                    "connection", &connection,
                    "id", &id,
                    "name", &name,
                    NULL);
            g_object_get (G_OBJECT (connection),
                    "host", &host,
                    "port", &port,
                    "username", &username,
                    "password", &password,
		    "database-id", &database_id,
		    "session-id", &session_id,
                    NULL);
            if (username && password) {
                uri = g_strdup_printf ("dpap://%s:%s@%s:%d/databases/%d/items?session-id=%d&meta=dpap.hires&query=('dmap.itemid:%d')&name=%s",
                        username, password,
                        host, port,
			database_id, session_id,
                        id,
                        name);
                g_free (username);
                g_free (password);
            }
            else {
                uri = g_strdup_printf ("dpap://%s:%d/databases/%d/items?session-id=%d&meta=dpap.hires&query=('dmap.itemid:%d')&name=%s",
                        host, port,
			database_id, session_id,
                        id,
                        name);
            }
            g_free (host);
            g_free (name);
            g_object_unref (connection);
            g_value_set_string (value, uri);
            g_free (uri);
        }
        break;

    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}
