/*
 * Copyright (C) 2004,2005 Charles Schmidt <cschmidt2@emich.edu>
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "dpap-root-item.h"
#include "dpap-album.h"
#include "dmap-connection.h"
#include "dmap-structure.h"

#include <libsoup/soup.h>
#include <libsoup/soup-session-sync.h>
#include <libsoup/soup-uri.h>

static gboolean dpap_root_item_retrieve_albums (DMAPItem *self);
static void root_album_loaded (DMAPItem *playlist,
                               DMAPItem *self);
static void handle_albums (DMAPConnection *connection,
                           guint status,
                           GNode *structure,
                           DMAPItem *self);

G_DEFINE_TYPE (DPAPRootItem, dpap_root_item, TYPE_DMAP_ITEM)

static void
dpap_root_item_class_init (DPAPRootItemClass *klass)
{
    DMAPItemClass *dmap_item_class = (DMAPItemClass *) klass;

    dmap_item_class->retrieve_items = dpap_root_item_retrieve_albums;
}

static void
dpap_root_item_init (DPAPRootItem *self)
{
}

static gboolean
dpap_root_item_retrieve_albums (DMAPItem *self)
{
    DMAPConnection *connection;
    gchar *path;
    gboolean ret;
    gint database_id, session_id;

    g_object_get (G_OBJECT (self),
            "connection", &connection,
	    "database-id", &database_id,
	    "session-id", &session_id,
            NULL);

    path = g_strdup_printf ("/databases/%d/containers?session-id=%d",
		database_id, session_id);
    ret = dmap_connection_get (connection, path,
            TRUE, (DMAPResponseHandler) handle_albums, self);
    g_free (path);
    g_object_unref (connection);

    return ret;
}

static void
root_album_loaded (DMAPItem *album,
                   DMAPItem *self)
{
    g_signal_handlers_disconnect_by_func (album,
            G_CALLBACK (root_album_loaded),
            self);
    dmap_item_signal_loaded (self);
}

static void
handle_albums (DMAPConnection *connection,
               guint status,
               GNode *structure,
               DMAPItem *self)
{
    GNode *listing_node;
    gint i;
    GNode *n;
    DMAPItem *root_album, *album;

    if (structure == NULL || SOUP_STATUS_IS_SUCCESSFUL (status) == FALSE) {
        return;
    }

    listing_node = dmap_structure_find_node (structure, DMAP_CC_MLCL);
    if (listing_node == NULL) {
        gint database_id;
        g_object_get (connection, "database-id", &database_id, NULL);
        g_debug ("Could not find dmap.listing item in /databases/%d/containers",
		 database_id);
        return;
    }

    root_album = DMAP_ITEM (g_object_new (TYPE_DPAP_ALBUM,
            "connection", connection,
            "id", -1,
            "name", "All",
            "is-folder", TRUE,
            NULL));
    g_signal_connect (root_album,
                      "loaded",
                      G_CALLBACK (root_album_loaded),
                      self);
    dmap_item_add_child (self, root_album);

    for (i = 0, n = listing_node->children; n; n = n->next, i++) {
        DMAPStructureItem *item;
        gint item_id;
        const gchar *name;

        item = dmap_structure_find_item (n, DMAP_CC_ABPL);
        if (item != NULL) {
            continue;
        }

        item = dmap_structure_find_item (n, DMAP_CC_MIID);
        if (item == NULL) {
            gint database_id;
            g_object_get (connection, "database-id", &database_id, NULL);
            g_debug ("Could not find dmap.itemid item in /databases/%d/containers",
	    	     database_id);
            continue;
        }
        item_id = g_value_get_int (&(item->content));

        item = dmap_structure_find_item (n, DMAP_CC_MINM);
        if (item == NULL) {
            gint database_id;
            g_object_get (connection, "database-id", &database_id, NULL);
            g_debug ("Could not find dmap.itemname item in /databases/%d/containers",
	    	     database_id);
            continue;
        }
        name = g_value_get_string (&(item->content));

        g_debug ("Got album with id %d, name %s", item_id, name);

        album = DMAP_ITEM (g_object_new (TYPE_DPAP_ALBUM,
                "connection", connection,
                "id", item_id,
                "name", name,
                "parent", root_album,
                "is-folder", TRUE,
                NULL));
        dmap_item_add_child (self, album);
    }

    dmap_item_retrieve_items (root_album, FALSE);
}
